<?php

namespace Beo\Testimonial\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Beo\Testimonial\Forms\Testimonial as TestimonialForm;
use Beo\Testimonial\Http\Requests\TestimonialRequest;
use Beo\Testimonial\Http\Resources\TestimonialResource;
use Beo\Testimonial\Http\Resources\TestimonialsCollection;
use Beo\Testimonial\Models\Testimonial;
use Beo\Testimonial\Scopes\TestimonialResourceScope;


/**
 * Resource controller class for testimonial.
 */
class TestimonialResourceController extends BaseController
{

    /**
     * Initialize testimonial resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = TestimonialForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('beo.testimonial.modules'), 'testimonial', guard_url('testimonial'));
    }

    /**
     * Display a list of testimonial.
     *
     * @return Response
     */
    public function index(TestimonialRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Testimonial::pushScope(new RequestScope())
            ->pushScope(new TestimonialResourceScope())
            ->simplePaginate($pageLimit);

        $data = new TestimonialsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('testimonial::testimonial.names'))
            ->view('testimonial::testimonial.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display testimonial.
     *
     * @param Request $request
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function show(TestimonialRequest $request, Testimonial $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestimonialResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('testimonial::testimonial.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('testimonial::testimonial.show')
            ->output();

    }

    /**
     * Show the form for creating a new testimonial.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TestimonialRequest $request, Testimonial $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestimonialResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('testimonial::testimonial.name'))
            ->view('testimonial::testimonial.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new testimonial.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TestimonialRequest $request, Testimonial $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new TestimonialResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('testimonial::testimonial.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('testimonial/testimonial/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/testimonial/testimonial'))
                ->redirect();
        }

    }

    /**
     * Show testimonial for editing.
     *
     * @param Request $request
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function edit(TestimonialRequest $request, Testimonial $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TestimonialResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('testimonial::testimonial.name'))
            ->view('testimonial::testimonial.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the testimonial.
     *
     * @param Request $request
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function update(TestimonialRequest $request, Testimonial $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new TestimonialResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('testimonial::testimonial.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('testimonial/testimonial/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('testimonial/testimonial/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the testimonial.
     *
     * @param Model   $testimonial
     *
     * @return Response
     */
    public function destroy(TestimonialRequest $request, Testimonial $model)
    {
        try {
            $model->delete();
            $data = new TestimonialResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('testimonial::testimonial.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('testimonial/testimonial/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('testimonial/testimonial/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
